# APRIORI_PYTHON_HEADER_START#################################################
#                                                          vi: set ts=8: sw=4:
# @(#) $Id: HtmlArea.py,v 1.13 2003/10/11 08:51:48 th Exp $
#
# This file is a part of the 'HtmlArea3 for Zope' product.
# ----------------------------------------------------------------------------
# Copyright (C) 2003  APRIORI business solutions AG
#                     http://www.apriori.de
# ----------------------------------------------------------------------------
#
# $Log: HtmlArea.py,v $
# Revision 1.13  2003/10/11 08:51:48  th
# ZPT support
#
# Revision 1.12  2003/10/09 02:43:21  th
# removeformat etc.
#
# Revision 1.10  2003/10/02 11:46:13  th
# initial copylefted version
#
#
#
# APRIORI_PYTHON_HEADER_END###################################################
import ExtensionClass
from Globals import DTMLFile
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

def htmlArea_version(self, REQUEST=None):
    "returns a string containing the underlying HtmlArea version"
    return 'HtmlArea 3.0 alpha for Zope'

# map the old button keys (HtmlArea 2.03) to the new ones
# for backward compatibility
oldButtonsMap = { # v2.03 name,      v3 name
                  'OrderedList':    'insertorderedlist',
                  'UnOrderedList':  'insertunorderedlist',
                  'Outdent':        'outdent',
                  'Indent':         'indent',
                  'fontstyle':      'formatblock',
                  'HorizontalRule': 'inserthorizontalrule',
                  'Createlink':     'createlink',
                  'InsertImage':    'insertimage',
                  'InsertTable':    'inserttable',
                  'HtmlMode':       'htmlmode',
                  'backcolor':      'hilitecolor',
                  'help':           'showhelp',
                 }


HtmlArea3_languages = [] # filled by product init code

def htmlArea_defaultLanguage(self, REQUEST=None, lang=None):
    """
    returns the first language from the visitor's browser
    preferences which is supported by this version of
    HtmlArea3 for Zope (i.e. a htmlarea-lang-xy.js file can
    be found).

    Arguments:

    **REQUEST** --
        For calls from DTML, should be inserted automagically

    **lang** --
        *default* language (a 'lang' value in the REQUEST will override this!)
    """
    if REQUEST is None:
        REQUEST = self.REQUEST
        if REQUEST is None:
            raise 'Ooops!', 'REQUEST is None'
    langs = (
              (REQUEST.get('lang') is not None and '%(lang)s,' % REQUEST) or
              (lang and '%s,' % lang) or
              ''
            ) + (
              REQUEST.get('HTTP_ACCEPT_LANGUAGE', 'en').strip() or 'en'
            )
    '''
    langs = REQUEST.get('HTTP_ACCEPT_LANGUAGE', 'en').strip() or 'en'
    #from HtmlArea import HtmlAreaMixin
    '''
    test = HtmlArea3_languages
    for lang in langs.split(','):
        l = lang.split(';', 1)[0].lower().replace('_', '-')
        if l in HtmlArea3_languages:
            return l
        if l.find('-') > -1:
            l = l.split('-')[0]
            if l in HtmlArea3_languages:
                return l
    return 'en'

def htmlArea_productLanguages(self):
    """
    returns the supported languages
    """
    return HtmlArea3_languages

htmlArea_prefix = DTMLFile('dtml/htmlArea_prefix', globals(),
                           lang=None) # use the first language of
                                      # the user's preferred which we support
htmlArea_head = DTMLFile('dtml/head', globals())

def htmlArea_allStatement(self):
    """
    Returns the Javascript (ECMAScript) statement which turns all textareas
    in the page into (equal) WYSIWYG editors; Customisation is only possible
    via specification of a new 'config' variable (not yet supported).

    Called by -> htmlArea_all(), or you can use it e.g. in the onload
    event handler of the page.
    """
    return 'HTMLArea.replaceAll();'


# don't rely on the numeric values, they may change
# (although the level names should remain in order):
htmlArea_toolbarLevels = {
    'empty':            0,
    'simple':           10,
    'restricted':       20,
    'common':           30,
    'common+clipboard': 40,
    'normal':           50,
    'standard':         60,
    'all':              100,
    'future':           255,  # --> HtmlArea 3 beta
    }

htmlArea_toolbarDescriptions = {
    'empty':        'unless at least one button switched on, yields standard',
    'simple':       'very basic character formatting',
    'restricted':   'basic character and text formatting',
    'common':       'a reasonable collection',
    # 'common+clipboard': 'a reasonable collection+clipboard',
    'normal':       'what InteractiveTools considers normal (configured)',
    'standard':     'not configured at all',
    'all':          'bells-and-whistles version',
    }

def _s2l(s): # string to level
    return htmlArea_toolbarLevels[s]

def htmlArea_selectLanguage(self, default):
    """
    a select tag, e.g. used by the demo
    """
    res=['<select name="lang" onchange="theForm.submit();">',
         '<option value="">(autodetect)</option>']
    for l in htmlArea_productLanguages(self):
        if l == default:
            res.append('<option selected="selected">%s</option>' % l)
        else:
            res.append('<option>%s</option>' % l)
    res.append('</select>')
    return ''.join(res)
    
def htmlArea_selectToolbar(self, default):
    """
    a select tag for the demo
    """
    tmp = []
    for k in htmlArea_toolbarDescriptions.keys():
        tmp.append({ 'key': k,
                     'descr': htmlArea_toolbarDescriptions[k],
                     'level': htmlArea_toolbarLevels.get(k, -1),
                     'selected': (k == default) and ' selected="selected"' or ''
                     })
    tmp.sort(lambda a, b: cmp(a['level'], b['level']))
    res=['<select name="toolbar" onchange="theForm.submit();">']
    for dic in tmp:
        res.append('<option value="%(key)s"'
                            '%(selected)s>'
                            '%(key)s (%(descr)s)'
                            '</option>' % dic)
    res.append('</select>')
    return ''.join(res)
        
# all known buttons, and their toolbar levels
htmlArea_buttonsAndToolbars = { # --> htmlArea_toolbarLevels
    'bold':         _s2l('simple'),
    'italic':       _s2l('simple'),
    'underline':    _s2l('simple'),
    'undo':         _s2l('simple'),
    'redo':         _s2l('simple'),
    'about':        _s2l('simple'),
    'justifyleft':      _s2l('restricted'),
    'justifyright':     _s2l('restricted'),
    'justifycenter':    _s2l('restricted'),
    'justifyfull':      _s2l('restricted'),
    'formatblock':          _s2l('common'),
    'subscript':            _s2l('common'),
    'superscript':          _s2l('common'),
    'insertorderedlist':    _s2l('common'),
    'insertunorderedlist':  _s2l('common'),
    'textindicator':        _s2l('common'),
    'createlink':           _s2l('common'),
    'cut':          _s2l('future'),
    'copy':         _s2l('future'),
    'paste':        _s2l('future'),
    'fontname':             _s2l('normal'),
    'fontsize':             _s2l('normal'),
    'strikethrough':        _s2l('normal'),
    'outdent':              _s2l('normal'),
    'indent':               _s2l('normal'),
    'linebreak':            _s2l('normal'),
    'forecolor':            _s2l('normal'),
    'hilitecolor':          _s2l('normal'),
    'inserthorizontalrule': _s2l('normal'),
    'insertimage':          _s2l('normal'),
    'inserttable':          _s2l('normal'),
    'htmlmode':             _s2l('normal'),
    'showhelp':             _s2l('normal'), # now filled with useful content(?)
    'popupeditor':          _s2l('normal'),
    'about':                _s2l('normal'),
    'removeformat':         _s2l('all'),    # not supported by original HtmlArea3 alpha
}

def htmlArea_all(self):
    """
    returns a script which turns all textareas in the page into
    WYSIWYG editors. Alternatively, you can put the call to
    'htmlArea_allStatement' in the onload event of your page
    or any other place you want.
    """
    return '\n'.join(('<script type="text/javascript" defer="1"><!--',
                      htmlArea_allStatement(self),
                      '//--></script>'))

def _htmlArea_selectButtons(self, set):
    """
    returns a dictionary of all buttons which are explicitely contained
    in the specified set. For an unknown set, None is returned;
    for 'all', the calling code uses the default value 1
    """
    lv = htmlArea_toolbarLevels.get(set, None)
    if lv is None:
        return None # unknown set
    dic = {}
    for k in htmlArea_buttonsAndToolbars.keys():
        dic[k] = (htmlArea_buttonsAndToolbars[k] <= lv)
    if not set in ('normal', 'standard', 'empty'):
        dic['removeformat'] = 1
    return dic

htmlArea_selectButtons = _htmlArea_selectButtons
def htmlArea_editor(self,
                    name, # name of the textarea
                    toolbar='common',
                    width=None,
                    height=None,
                    **buttons):
    """
    Inserts the code to turn the textarea with the given *name*
    into a WYSIWYG editor. Must be called in the html body;
    requires the use of --> htmlArea_prefix in the html head.

    Arguments:

    **name** --
        the name and id of the textarea

    **height** --
        a height specification for the editor (px or percentage);
        default: '200px' (may change).

        Note:

              when neither *height* nor *width* nor any toolbar
              modification is specified, just the standard
              editor is created, without any *config* specification!
              This might lead to unexpected results.

    **width** --
        a width specification for the editor (px or percentage);
        default: '100%' (may change)

    **toolbar** --
        allows the preselection of a set of available buttons.
        Currently supported (string values):

             *simple* --
                bold, italic, underline, about only

             *restricted* --
                adds justify buttons and links

             *normal* --
                adds lists, indentations and more
                (includes *restricted*); almost the same as
                *standard*, but always defines the toolbar
                explicitely

             *standard* --
                what htmlArea 3 considers standard
                (doesn't explicitely define the toolbar unless
                specific buttons requested, see below)

             *common* --
                a set of buttons reasonable for a site which
                tries to avoid typographic noise but instead
                is designed using stylesheets (CI/CD).
             
             *all* --
                all standard buttons (of HtmlArea 3) known to
                this Zope product
                 
    ****buttons** --
        allows to switch on or off single buttons
        by specifying named arguments. Any htmlArea 2 button keys
        are translated to the new names, so you won't really need to
        change your code.

        For the possible keys, have a look at your language file,
        e.g. "/misc_/HtmlArea3/htmlarea-lang-en.js":/misc_/HtmlArea3/htmlarea-lang-en.js
    """
    if not (height or width or (toolbar != 'standard') or buttons):
        return """\n<script language="javascript1.2"><!--
HTMLArea.replace('%s');
//--></script>""" % name

    # defer: empty attribute, prevents the script from generating code; newer MSIEs only
    res = """\n<script language="javascript1.2" defer><!--\n"""
    
    tint = type(0)
    if height is None:
        height = '200px'
    elif type(height) == tint:
        height = '%dpx' % height
    
    if width is None:
        width = '100%'
    elif type(width) == tint:
        if width > 100:
            width = '%dpx' % width
        else:
            width = '%d%%' % width

    if toolbar == 'all':
        _toolbar = { 'showhelp': 1, # reference
                     # 'formatblock': 0, # fontstyle didn't work in 2.03
                    }
        default = 1
    elif buttons or (toolbar != 'standard'):
        if toolbar == 'standard':
            toolbar = 'normal'
        _toolbar = htmlArea_selectButtons(self, toolbar)
        if _toolbar is None:
            res += '// unknown toolbar set: "%s";\n' % toolbar
            toolbar = buttons and 'normal' or 'standard'
            res += '// changed to: "%s"\n' % toolbar
            _toolbar = htmlArea_selectButtons(self, toolbar)
        if _toolbar:
            default = 0
        else:
            default = 1
    if buttons:
        for k in buttons.keys():
            tmp = oldButtonsMap.get(k)
            if tmp is not None:
                # don't override a new key with an old one:
                if buttons.get(tmp) is None:
                    _toolbar[tmp] = buttons[k]
            else:
                _toolbar[k] = buttons[k]

    res += 'var config = new HTMLArea.Config();\n'
    if width:
        res += "config.width = '%s';\n" % width
    if height:
        res += "config.height = '%s';\n" % height
    lines = []
    bold_done = 0

    if buttons or (toolbar != 'standard'):
        tmpline = []
        for keyset in ( ('fontname', 'fontsize', 'formatblock',
                         'bold', 'italic', 'underline',),
                        ('strikethrough', 'subscript', 'superscript',),
                        ('removeformat',),
                        ('linebreak',),
                        ('justifyleft', 'justifycenter', 'justifyright', 
                         'justifyfull'), # new in htmlArea 3
                        ('insertorderedlist', 'insertunorderedlist', 'outdent', 'indent',),
                        ('forecolor', 'hilitecolor', 'textindicator',),
                        ('inserthorizontalrule', 'createlink', 'insertimage', 'inserttable'),
                        ('htmlmode', 'popupeditor',),
                        ('about', 'showhelp',),
                       ):
            for key in keyset:
                if _toolbar.get(key, default):
                    # first insert spaces left to certain controls...
                    if bold_done:
                        pass
                    elif key in ('fontsize', 'formatblock'):
                        if tmpline:
                            tmpline.append('space')
                    elif key in ('bold', 'italic', 'underline',
                                 'strikethrough', 'subscript', 'superscript',
                                 'removeformat'):
                        bold_done = 1
                        if tmpline:
                            tmpline.append('space')
                    # ... then append the really interesting key:
                    tmpline.append(key)

            if tmpline:
                if tmpline[-1] != 'linebreak':
                    tmpline.append('separator')
            if tmpline: # don't append empty lists!
                lines.append(tmpline) # lines is a list of lists
                # lines.append("'%s'" % "', '".join(tmpline))
                tmpline = []
        
    if lines:
        for li in range(1, len(lines)):
            if lines[li] and lines[li][0] == 'linebreak':
                if lines[li-1] and lines[li-1][-1] == 'separator':
                    del lines[li-1][-1]
        if lines[-1][-1] in ('linebreak', 'separator'):
            del lines[-1][-1]
            if not lines[-1]:
                del lines[-1]
    if lines:
        res += 'config.toolbar = [[%s]];\n' % \
               "],\n                  [".join(["'%s'" % "', '".join(t)
                                               for t in lines
                                               if t ])

    res += "HTMLArea.replace('%s', config);\n" % name
    res += '//--></script>'
    return res

# NOT YET TESTED:
def htmlArea(self,
             name,              # name of the textarea
             val='',            # additional here
             toolbar='common',
             width=None,
             height=None,
             cols=0,            # additional here;
             rows=0,            # defaults might change!
             **buttons):
    """
    **NOT YET FINISHED**

    Creates both the textarea and the Javascript
    transformation code.

    Arguments:

    **name** --
        the name and id of the textarea. Propagated to htmlArea_editor.

    **val** --
        this *applies only to the textarea;*
        the initial value filled into the textarea

    **toolbar**
        please refer to -> 'htmlArea_editor'

    **height** --
        a height specification for the textarea and the editor
        (px or percentage);
        default: '200px' (may change;
        you may specify *None* to use the default).

        Please take note of the remarks to the -> htmlArea_editor function.

    **width** --
        a width specification for the textarea and the editor
        (px or percentage);
        default: '100%' (may change;
        you may specify *None* to use the default).

    **cols** --
        if given, must be a positive integer.
        Default: None (might change);
        reasonable non-None value: 60.
        *(applies only to the textarea)*

    **rows** --
        if given, must be a positive integer.
        Default: None (might change);
        reasonable non-None value: 6.
        *(applies only to the textarea)*

    ****buttons** --
        please refer to -> 'htmlArea_editor'
    """
    # the usual code... (please sync!)
    tint = type(0)
    if height is None:
        height = '200px'
    elif type(height) == tint:
        height = '%dpx' % height
    
    if width is None:
        width = '100%'
    elif type(width) == tint:
        if width > 100:
            width = '%dpx' % width
        else:
            width = '%d%%' % width
    # ... the usual code
    
    if 1:
        dic = {
                'name': name,
                'id': name,
                'rows': rows and ('rows="%s"' % rows) or '',
                'cols': cols and (' cols="%s"' % cols) or '',
                'width': width and ' width="%s"' % width or '',
                'height': height and ' height="%s"' % height or '',
                'val': val,
               }
        res = ['<textarea',]
        res.append(  'name="%(name)s"' % dic )
        res.append('id="%(id)s"' % dic)
        res.append('%(rows)s%(cols)s%(width)s%(height)s>%(val)s</textarea>\n' % dic)
        res = ' '.join(res)
    else:
        res = ' '.join(('<textarea',
                    'name="%(name)s"',
                    'id="%(id)s"',
                    '%(rows)s%(cols)s%(width)s%(height)s>%(val)s</textarea>\n',
                    )) % { 'name': name,
                           'id': name,
                           'rows': rows and ('rows="%d"' % rows) or '',
                           'cols': cols and (' cols="%d"' % cols) or '',
                           'width': width and ' width="%s"' % width or '',
                           'height': height and ' height="%s"' % height or '',
                           'val': val,
                          }
    res += htmlArea_editor(self,
                           name=name,
                           toolbar=toolbar,
                           width=width,
                           height=height,
                           **buttons)
    return res

def htmlArea_style(self):
    """
    simply includes the standard styles
    """
    return '<link rel="stylesheet" type="text/css" href="/misc_/HtmlArea3/htmlarea.css">'

htmlArea_demo=DTMLFile('dtml/demo', globals())
htmlArea_ = PageTemplateFile('zpt/htmlArea_.pt', globals())
    
class HtmlAreaMixin(ExtensionClass.Base):
    """
    A mixin class just to provide the DTML method HtmlArea_script.
    It doesn't define an own object type you could create objects of.

    You don't *really* need to use the mixin class (although this
    is recommended); you may as well simply insert the contents of
    the method into the method[s] you intend to use.
    
    Methods:

    **htmlArea_editor** --
        creates an editor for the specified textarea,
        several options

    **htmlArea** --
        creates an editor *including* the textarea;
        calls -> 'htmlArea_editor'

    **htmlArea_all** --
        transforms all textareas in the page,
        without further config.options

    **htmlArea_allStatement** --
        the Javascript statement used by -> 'htmlArea_all'

    **htmlArea_prefix** --
        includes the editor.js if the browser
        is recognised as a supported one
        (MSIE/Win >= 5.5, or Mozilla >= 1.3).
        You may specify the language for the button
        tooltips, e.g. lang='de'; by default,
        the user's preferences are used

    **htmlArea_defaultLanguage** --
        returns the standard language used e.g. for 
        the buttons tooltips, according to browser settings
        and the installed language files (htmlarea-lang-xx.js).
        Called by -> 'htmlArea_prefix'
    
    **htmlArea_style** --
        creates a html link to include the standard stylesheet
    """
    htmlArea_prefix         = htmlArea_prefix
    htmlArea                = htmlArea
    htmlArea_editor         = htmlArea_editor
    htmlArea_style          = htmlArea_style
    htmlArea_head           = htmlArea_head
    htmlArea_defaultLanguage= htmlArea_defaultLanguage
    htmlArea_productLanguages=htmlArea_productLanguages 
    htmlArea_all            = htmlArea_all
    htmlArea_allStatement   = htmlArea_allStatement 
    htmlArea_version        = htmlArea_version
    htmlArea_selectToolbar  = htmlArea_selectToolbar
    htmlArea_selectLanguage = htmlArea_selectLanguage
    htmlArea_demo           = htmlArea_demo
    htmlArea_               = htmlArea_
